#include <Python.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#define VERSION "0.0"
#define MODULE "_functional"
static PyObject *moduleError;
static PyObject *moduleVersion;
static char *moduleDoc =
"_functional is a Python extension that defines some simple C accelerations for\n\
use with map et al.\n\
";

typedef struct {
	PyObject_HEAD
	int	kind,start,stop,step;
	} pySlicerObject;

static void pySlicerFree(pySlicerObject* self)
{
	PyMem_DEL(self);
}

static struct PyMethodDef pySlicer_methods[] = {
	{NULL, NULL}		/* sentinel */
};

static PyObject* pySlicer_getattr(pySlicerObject *self, char *name)
{
	if(!strcmp(name,"start")) return PyInt_FromLong(self->start);
	return Py_FindMethod(pySlicer_methods, (PyObject *)self, name);
}

static PyObject* pySlicer_call(pySlicerObject *self, PyObject* args)
{
	PyObject*	seq;
	if(!PyArg_ParseTuple(args, "O:Slicer.__call__", &seq)) return NULL;
	return PySequence_GetItem(seq,self->start);
}

static PyTypeObject pySlicerType = {
	PyObject_HEAD_INIT(0)
	0,								/*ob_size*/
	"SlicerType",					/*tp_name*/
	sizeof(pySlicerObject),		/*tp_basicsize*/
	0,								/*tp_itemsize*/
	/* methods */
	(destructor)pySlicerFree,		/*tp_dealloc*/
	(printfunc)0,					/*tp_print*/
	(getattrfunc)pySlicer_getattr,	/*tp_getattr*/
	(setattrfunc)0,					/*tp_setattr*/
	(cmpfunc)0,						/*tp_compare*/
	(reprfunc)0,					/*tp_repr*/
	0,								/*tp_as_number*/
	0,								/*tp_as_sequence*/
	0,								/*tp_as_mapping*/
	(hashfunc)0,					/*tp_hash*/
	(ternaryfunc)pySlicer_call,	/*tp_call*/
	(reprfunc)0,					/*tp_str*/

	/* Space for future expansion */
	0L,0L,0L,0L,
	/* Documentation string */
	"Slicer instance see module __doc__"
};

static pySlicerObject* pySlicer(PyObject* module, PyObject* args)
{
	pySlicerObject* self;
	PyObject *start, *stop, *step;
	int istart, istop, istep, kind;
	start = stop = step = NULL;

	if (!PyArg_ParseTuple(args, "O|OO:Slicer", &start, &stop, &step)) return NULL;
	if(PyInt_Check(start)){
		PyErr_SetString(moduleError,"Need at least one integer argument!");
		return NULL;
		}

	if(!(self = PyObject_NEW(pySlicerObject, &pySlicerType))) return NULL;
	if(!stop || stop==PyNone){
		self->kind = 0;
		}
	else {
		}
	return self;
}

static struct PyMethodDef moduleMethods[] = {
	{"Slicer",	(PyCFunction)pySlicer,	METH_VARARGS, "Slicer(start[,stop[increment]]) create a Slicer instance"},
	{NULL,	NULL}	/*sentinel*/
};

DL_EXPORT(void) init_functional(void)
{
	PyObject *m, *d, *v;

	/*set up the types by hand*/
	pySlicerType.ob_type = &PyType_Type;

	/* Create the module and add the functions */
	m = Py_InitModule(MODULE, moduleMethods);

	/* Add some symbolic constants to the module */
	d = PyModule_GetDict(m);
	moduleVersion = PyString_FromString(VERSION);
	PyDict_SetItemString(d, "version", moduleVersion );
	moduleError = PyErr_NewException(MODULE ".Error",NULL,NULL);
	PyDict_SetItemString(d,"error",moduleError);
	
	/*add in the docstring*/
	v = PyString_FromString(moduleDoc);
	PyDict_SetItemString(d, "__doc__", v);
	Py_DECREF(v);
}
